
#include "file.bi"

#include "utils.bas"
#include "clreader.bas"

const OPTION_UPCASE = ""
const OPTION_LOWCASE = "-l"
const OPTION_CAPITALIZED = "-c"
const OPTION_INDIVIDUAL = "-i"
const DEFAULT_OPTION = OPTION_UPCASE
const BAS = ".bas"
const BI = ".bi"

dim shared flog as integer
flog = FreeFile()

dim fbeauty as string = ExePath & "\" & "fbeauty.exe"
dim filename as string = "" ' file or folder name
dim style as string = DEFAULT_OPTION
dim options as TCommandLine

open ExePath & "\" & "beautify-log.txt" for output as #flog
print #flog, Time

if options.HasOption("fbeauty") then
  fbeauty = options.GetOptionValue("fbeauty")
end if

if not FileExists(fbeauty) then
  print #flog, "Cannot find """ & LCase(fbeauty) & """"
  close #flog
  end
end if

if not (Right(fbeauty, Len("fbeauty.exe")) = "fbeauty.exe") then
  print #flog, "Invalid option value: """ & LCase(fbeauty) & """"
  print #flog, "The value expected is the path to fbeauty.exe"
  close #flog
  end
end if

if options.HasOption("file") then
  filename = options.GetOptionValue("file")
elseif options.HasOption("directory") then
  filename = options.GetOptionValue("directory")
elseif options.GetAloneValueNth(1) <> "" then
  filename = options.GetAloneValueNth(1)
end if

if filename = "" then
  print #flog, "Missing file name"
  close #flog
  end
end if

if not (FileExists(filename) or DirExists(filename)) then
  print #flog, "Invalid file name: """ & LCase(filename) & """"
  close #flog
  end
end if

if options.HasOption("style") then
  select case options.GetOptionValue("style")
    case "upcase"
      style = OPTION_UPCASE
    case "lowcase"
      style = OPTION_LOWCASE
    case "capitalized"
      style = OPTION_CAPITALIZED
    case "individual"
      style = OPTION_INDIVIDUAL
  end select
end if

print #flog, "fbeauty path: """ & LCase(fbeauty) & """"
print #flog, "file or folder name: """ & LCase(filename) & """"
print #flog, "style: """ & style & """"

declare sub BeautifyFile(byval as string, byval as string, byval as string)
declare sub BeautifyDirectory(byval as string, byval as string, byval as string)

if DirExists(filename) then
  if FindExt(filename, BAS) _
  or FindExt(filename, BI) then
    BeautifyDirectory(fbeauty, filename & "\", style)
  else
    print #flog, "No FreeBASIC source file found in the directory"
  end if
elseif FileExists(filename) then
  if (LCase(ExtractFileExt(filename)) = BAS) _
  or (LCase(ExtractFileExt(filename)) = BI) then 
    BeautifyFile(fbeauty, filename, style)
  else
    print #flog, "The file isn't a FreeBASIC source file"
  end if
end if

close #flog

sub Filter(byval aSource as string, byval aNewFile as string, byval aFilter as string, byval aOption as string)
  dim s as string
  dim as uinteger f1, f2
  f1 = FreeFile()
  open aNewFile for output as #f1
  f2 = FreeFile()
  open pipe aFilter & " " & aOption & " < " & aSource for input as #f2
  do until Eof(f2)
    line input #f2, s
    print #f1, s
  loop
  close #f2
  close #f1
  print #flog, "Beautified """ & LCase(aNewFile) & """"
end sub

sub BeautifyFile(byval aFilter as string, byval aPath as string, byval aOption as string)
  const TEMP = ".temp"
  dim s as string = ChangeFileExt(aPath, TEMP)
  if Name(aPath, s) = 0 then
    Filter(s, aPath, aFilter, aOption)
    Kill(s)
  else
    print #flog, "Error renaming """ & aPath & """ to """ & s & """"
  end if
end sub

sub BeautifyDirectory(byval aFilter as string, byval aPath as string, byval aOption as string)
  const PRESERVE_CONTENT = -1
  dim a() as string
  GetExt(aPath, BAS, a())
  GetExt(aPath, BI, a(), PRESERVE_CONTENT)
  for i as integer = LBound(a) to UBound(a)
    BeautifyFile(aFilter, aPath & a(i), aOption)
  next i
end sub
